"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const code_frame_1 = require("@babel/code-frame");
const generator_1 = __importDefault(require("@babel/generator"));
const cli_highlight_1 = require("cli-highlight");
const xml_ui_compiler_1 = require("@nativescript-community/xml-ui-compiler");
const path_1 = require("path");
const util_1 = require("util");
function loader(content, map) {
    const callback = this.async();
    const options = this.getOptions();
    // Error and warning handling
    (0, xml_ui_compiler_1.setAbnormalStateReceiveListener)((type, msg, range) => {
        const error = new Error(getCodeFrame(content, msg, range));
        if (type === xml_ui_compiler_1.AbnormalState.ERROR) {
            if (options.compileWithMarkupErrors) {
                this.emitError(error);
            }
            else {
                throw error;
            }
        }
        else if (type === xml_ui_compiler_1.AbnormalState.WARNING) {
            this.emitWarning(error);
        }
        else {
            throw error;
        }
    });
    loadContent(this, content, options).then((output) => {
        callback(null, output, map);
    }).catch((err) => {
        callback(err);
    });
}
exports.default = loader;
function getCodeFrame(content, msg, range) {
    const location = {
        start: range[0],
        end: range[1]
    };
    const codeFrame = (0, code_frame_1.codeFrameColumns)(content, location, {
        message: msg
    });
    return '\n' + (0, cli_highlight_1.highlight)(codeFrame, {
        language: 'xml'
    });
}
async function loadContent(loader, content, options) {
    const moduleRelativePath = (0, path_1.relative)(options.appPath, loader.resourcePath);
    const { output, pathsToResolve } = (0, xml_ui_compiler_1.compile)(content, moduleRelativePath, options.platform, options.useDataBinding, options.preprocess?.attributeValueFormatter);
    if (output == null) {
        return '';
    }
    // XML parser does not work asynchronously, so we resolve requested modules once everything is done
    if (pathsToResolve?.length) {
        const resolveAsync = (0, util_1.promisify)(loader.resolve);
        await Promise.all(pathsToResolve.map(request => resolveAsync(loader.context, request)));
    }
    // Convert AST to JS
    return options.preprocess?.transformAst ? options.preprocess?.transformAst(output, generator_1.default) : (0, generator_1.default)(output).code;
}
//# sourceMappingURL=index.js.map